@@ -17,16 +17,19 @@
 #include <QQuickRenderControl>
 #include <QQuickWindow>
 
-#include <KGlobalAccel>
-#include <KGlobalShortcutInfo>
 #include <KLocalizedString>
 #include <KMessageBox>
 #include <KStandardShortcut>
 
+#ifndef Q_OS_WIN
+#include <KGlobalAccel>
+#include <KGlobalShortcutInfo>
+#endif
+
 class KeySequenceHelperPrivate
 {
 public:
-    KeySequenceHelperPrivate(KeySequenceHelper *q);
+    KeySequenceHelperPrivate(KeySequenceHelper *qq);
 
     /**
      * Conflicts the key sequence @a seq with a current standard
@@ -62,8 +65,8 @@ public:
     KeySequenceHelper::ShortcutTypes checkAgainstShortcutTypes;
 };
 
-KeySequenceHelperPrivate::KeySequenceHelperPrivate(KeySequenceHelper *q)
-    : q(q)
+KeySequenceHelperPrivate::KeySequenceHelperPrivate(KeySequenceHelper *qq)
+    : q(qq)
     , checkAgainstShortcutTypes(KeySequenceHelper::StandardShortcuts | KeySequenceHelper::GlobalShortcuts)
 {
 }
@@ -117,24 +120,44 @@ bool KeySequenceHelperPrivate::conflictWithGlobalShortcuts(const QKeySequence &k
             "The F12 key is reserved on Windows, so cannot be used for a global shortcut.\n"
             "Please choose another one.");
 
-        KMessageBox::sorry(nullptr, message, title);
-        return false;
+        KMessageBox::error(nullptr, message, title);
     }
-#endif
-
+    return false;
+#else
     if (!(checkAgainstShortcutTypes & KeySequenceHelper::GlobalShortcuts)) {
         return false;
     }
 
-    // Global shortcuts are on key+modifier shortcuts. They can clash with
-    // each of the keys of a multi key shortcut.
+    // Global shortcuts are on key+modifier shortcuts. They can clash with a multi key shortcut.
     QList<KGlobalShortcutInfo> others;
-    for (int i = 0; i < keySequence.count(); ++i) {
-        QKeySequence tmp(keySequence[i]);
+    QList<KGlobalShortcutInfo> shadow;
+    QList<KGlobalShortcutInfo> shadowed;
+    if (!KGlobalAccel::isGlobalShortcutAvailable(keySequence, QString())) {
+        others << KGlobalAccel::globalShortcutsByKey(keySequence);
 
-        if (!KGlobalAccel::isGlobalShortcutAvailable(tmp, QString())) {
-            others << KGlobalAccel::getGlobalShortcutsByKey(tmp);
+        // look for shortcuts shadowing
+        shadow << KGlobalAccel::globalShortcutsByKey(keySequence, KGlobalAccel::MatchType::Shadows);
+        shadowed << KGlobalAccel::globalShortcutsByKey(keySequence, KGlobalAccel::MatchType::Shadowed);
+    }
+
+    if (!shadow.isEmpty() || !shadowed.isEmpty()) {
+        QString title = i18n("Global Shortcut Shadowing");
+        QString message;
+        if (!shadowed.isEmpty()) {
+            message += i18n("The '%1' key combination is shadowed by following global actions:\n").arg(keySequence.toString());
+            for (const KGlobalShortcutInfo &info : std::as_const(shadowed)) {
+                message += i18n("Action '%1' in context '%2'\n").arg(info.friendlyName(), info.contextFriendlyName());
+            }
         }
+        if (!shadow.isEmpty()) {
+            message += i18n("The '%1' key combination shadows following global actions:\n").arg(keySequence.toString());
+            for (const KGlobalShortcutInfo &info : std::as_const(shadow)) {
+                message += i18n("Action '%1' in context '%2'\n").arg(info.friendlyName(), info.contextFriendlyName());
+            }
+        }
+
+        KMessageBox::error(nullptr, message, title);
+        return true;
     }
 
     if (!others.isEmpty() && !KGlobalAccel::promptStealShortcutSystemwide(nullptr, others, keySequence)) {
@@ -147,10 +170,9 @@ bool KeySequenceHelperPrivate::conflictWithGlobalShortcuts(const QKeySequence &k
     // error it just silently fails. So be nice because this is
     // most likely the first action that is done in the slot
     // listening to keySequenceChanged().
-    for (int i = 0; i < keySequence.count(); ++i) {
-        KGlobalAccel::stealShortcutSystemwide(keySequence[i]);
-    }
+    KGlobalAccel::stealShortcutSystemwide(keySequence);
     return false;
+#endif
 }
 
 bool KeySequenceHelperPrivate::conflictWithStandardShortcuts(const QKeySequence &keySequence)
@@ -182,6 +204,11 @@ bool KeySequenceHelperPrivate::stealStandardShortcut(KStandardShortcut::Standard
     return true;
 }
 
+QKeySequence KeySequenceHelper::fromString(const QString &str)
+{
+    return QKeySequence::fromString(str, QKeySequence::NativeText);
+}
+
 bool KeySequenceHelper::keySequenceIsEmpty(const QKeySequence &keySequence)
 {
     return keySequence.isEmpty();
@@ -196,7 +223,7 @@ QWindow *KeySequenceHelper::renderWindow(QQuickWindow *quickWindow)
 {
     QWindow *renderWindow = QQuickRenderControl::renderWindowFor(quickWindow);
     auto window = renderWindow ? renderWindow : quickWindow;
-    // If we have CppOwnership, set it explicitely to prevent the engine taking ownership of the window
+    // If we have CppOwnership, set it explicitly to prevent the engine taking ownership of the window
     // and crashing on teardown
     if (QQmlEngine::objectOwnership(window) == QQmlEngine::CppOwnership) {
         QQmlEngine::setObjectOwnership(window, QQmlEngine::CppOwnership);
